use futures::Sink;
use misskey::{model::note::Visibility, Client, ClientExt, HttpClient};
use std::{error::Error, task::Poll};
use url::Url;

pub struct MisskeyPublisher {
    client: HttpClient,
    post_visibility: Visibility,
}

impl MisskeyPublisher {
    pub fn new(url: &String, token: String, vis: Visibility) -> Result<Self, Box<dyn Error>> {
        Ok(Self {
            client: HttpClient::with_token(Url::parse(url)?, token)?,
            post_visibility: vis,
        })
    }
}

impl Sink<String> for MisskeyPublisher {
    type Error = Box<dyn Error>;

    fn poll_ready(
        self: std::pin::Pin<&mut Self>,
        _: &mut std::task::Context<'_>,
    ) -> std::task::Poll<Result<(), Self::Error>> {
        Poll::Ready(Ok(()))
    }

    fn start_send(self: std::pin::Pin<&mut Self>, item: String) -> Result<(), Self::Error> {
        let mut req = self.client.build_note();
        let req = req.text(item).visibility(self.post_visibility).as_request();
        let response = smol::block_on(self.client.request(req))?;
        // If there is an ApiResult error, decode it to a regular Result error
        if let Option::Some(err) = response.err() {
            return Err(Box::new(err));
        }
        Ok(())
    }

    fn poll_flush(
        self: std::pin::Pin<&mut Self>,
        _: &mut std::task::Context<'_>,
    ) -> std::task::Poll<Result<(), Self::Error>> {
        Poll::Ready(Ok(()))
    }

    fn poll_close(
        self: std::pin::Pin<&mut Self>,
        _: &mut std::task::Context<'_>,
    ) -> std::task::Poll<Result<(), Self::Error>> {
        Poll::Ready(Ok(()))
    }
}
